use std::mem;

use utilities::impl_reprc;
use vulkan_rs::prelude::*;

impl_reprc!(
    pub struct PositionOnlyVertex {
        #[assume_reprc]
        position: cgmath::Vector4<f32>,
    }
);

impl PositionOnlyVertex {
    ///
    /// corners[0]  -  bottom left
    /// corners[1]  -  top left
    /// corners[2]  -  top right
    /// corners[3]  -  bottom right
    ///
    pub fn from_2d_corners(
        ortho: cgmath::Matrix4<f32>,
        corners: [cgmath::Vector2<f32>; 4],
    ) -> [Self; 6] {
        [
            Self {
                position: ortho * corners[0].extend(0.0).extend(1.0),
            },
            Self {
                position: ortho * corners[1].extend(0.0).extend(1.0),
            },
            Self {
                position: ortho * corners[2].extend(0.0).extend(1.0),
            },
            Self {
                position: ortho * corners[2].extend(0.0).extend(1.0),
            },
            Self {
                position: ortho * corners[3].extend(0.0).extend(1.0),
            },
            Self {
                position: ortho * corners[0].extend(0.0).extend(1.0),
            },
        ]
    }
}

impl VertexInputDescription for PositionOnlyVertex {
    fn bindings() -> Vec<vulkan_rs::prelude::VkVertexInputBindingDescription> {
        vec![VkVertexInputBindingDescription {
            binding: 0,
            stride: mem::size_of::<PositionOnlyVertex>() as u32,
            inputRate: VK_VERTEX_INPUT_RATE_VERTEX,
        }]
    }

    fn attributes() -> Vec<vulkan_rs::prelude::VkVertexInputAttributeDescription> {
        vec![
            // position
            VkVertexInputAttributeDescription {
                location: 0,
                binding: 0,
                format: VK_FORMAT_R32G32B32A32_SFLOAT,
                offset: 0,
            },
        ]
    }
}
